@@ -1,25 +1,31 @@
 #include <glib.h>
+#include <gio/gio.h>
 #include <gcrypt.h>
 #include <jansson.h>
 #include <time.h>
-#include "../file-size.h"
-#include "../parse-uri.h"
-#include "../gquarks.h"
+#include "file-size.h"
+#include "gquarks.h"
+#include "parse-uri.h"
 
 
 GSList *
-get_freeotpplus_data (const gchar     *path,
-                      GError         **err)
+get_freeotpplus_data (const gchar  *path,
+                      gint32        max_file_size,
+                      GError      **err)
 {
     GSList *otps = NULL;
     goffset fs = get_file_size (path);
     if (fs < 10) {
-        g_printerr ("Couldn't get the file size (file doesn't exit or wrong file selected\n");
+        g_set_error (err, file_too_big_gquark (), GENERIC_ERRCODE, "Couldn't get the file size (file doesn't exit or wrong file selected.");
+        return NULL;
+    }
+    if (fs > max_file_size) {
+        g_set_error (err, file_too_big_gquark (), FILE_TOO_BIG, FILE_SIZE_SECMEM_MSG);
         return NULL;
     }
     gchar *sec_buf = gcry_calloc_secure (fs, 1);
     if (!g_file_get_contents (path, &sec_buf, NULL, err)) {
-        g_printerr("Couldn't read into memory the freeotp txt file\n");
+        g_printerr("Couldn't read into memory the freeotp txt file.\n");
         gcry_free (sec_buf);
         return NULL;
     }
@@ -44,8 +50,8 @@ export_freeotpplus (const gchar *export_path,
     GFileOutputStream *out_stream = g_file_replace (out_gfile, NULL, FALSE, G_FILE_CREATE_REPLACE_DESTINATION | G_FILE_CREATE_PRIVATE, NULL, &err);
     if (err == NULL) {
         json_array_foreach (json_db_data, index, db_obj) {
-            gchar *uri = get_otpauth_uri (NULL, db_obj);
-            if (g_output_stream_write (G_OUTPUT_STREAM(out_stream), uri, g_utf8_strlen (uri, -1) + 1, NULL, &err) == -1) {
+            gchar *uri = get_otpauth_uri (db_obj);
+            if (g_output_stream_write (G_OUTPUT_STREAM(out_stream), uri, g_utf8_strlen (uri, -1), NULL, &err) == -1) {
                 g_set_error (&err, generic_error_gquark (), GENERIC_ERRCODE, "couldn't dump json data to file");
             }
             g_free (uri);
